from portal import create_app
from portal import db
from portal.rest.auth.login import auth
from sample_data.data_init import init_data

from portal.rest.courses.courses import courses
from portal.rest.users.users import users
from portal.rest.roles.roles import roles
from portal.rest.groups.groups import groups
from portal.rest.submissions.submissions import submissions
from portal.rest.projects.projects import projects


def init_db(app):
    db.drop_all(app=app)
    db.create_all(app=app)


def register_blueprints(app):
    api_prefix = "/api/v1.0"
    app.register_blueprint(users, url_prefix=f"{api_prefix}/users")
    app.register_blueprint(courses, url_prefix=f"{api_prefix}/courses")
    app.register_blueprint(roles, url_prefix=f"{api_prefix}/courses/<string:cid>/roles")
    app.register_blueprint(groups, url_prefix=f"{api_prefix}/courses/<string:cid>/groups")
    app.register_blueprint(projects, url_prefix=f"{api_prefix}/courses/<string:cid>/projects")
    app.register_blueprint(submissions, url_prefix=f"{api_prefix}/submissions")
    app.register_blueprint(auth, url_prefix=f"{api_prefix}/auth")
    from portal.rest.auth.gitlab import oauth_blueprint
    app.register_blueprint(oauth_blueprint, url_prefix=f"{api_prefix}/oauth")


def setup_application():
    app = create_app()
    init_db(app)
    init_data(app, db)
    register_blueprints(app)
    return app


if __name__ == '__main__':
    app = setup_application()
    app.run(port=8000)

'''
# look into: flask-login extension (user authentication)
# flask-migrate
# use blueprints for the app - http://flask.pocoo.org/docs/0.12/blueprints/#blueprints
# flask-wtf for auth
# https://scotch.io/tutorials/build-a-crud-web-app-with-python-and-flask-part-one

logging: (email sending!):
http://flask.pocoo.org/docs/dev/logging/
https://docs.python.org/3/howto/logging-cookbook.html#formatting-styles

REST & js:
https://auth0.com/blog/developing-restful-apis-with-python-and-flask/
https://blog.miguelgrinberg.com/post/designing-a-restful-api-with-python-and-flask

Extensions for timestamping entities and _repr:
https://sqlalchemy-utils.readthedocs.io/en/latest/models.html#sqlalchemy_utils.models.Timestamp
'''