from portal.database.models import User, Course, Project, Group, Role, Submission, Review, ReviewItem
from portal.database import SubmissionState


def init_data(app, db):
    with app.app_context():
        create_users(db)
        create_courses(db)
        create_groups(db)
        create_projects(db)
        create_roles(db)
        create_submissions(db)
        create_reviews(db)
        create_relationships(db)


def create_users(db):
    foo_user = User(uco=445, email="foo@bar.cz", username="xfoo")
    foo_user.set_password('123456')
    foo_user.is_admin = True
    db.session.add(foo_user)
    bar_user = User(uco=123, email="bar@baz.cz", username="xbar")
    bar_user.set_password('123')
    db.session.add(bar_user)
    db.session.commit()


def create_courses(db):
    cpp = Course(name="C++", codename="PB161")
    cpp.notes_access_token = "token"
    db.session.add(cpp)
    db.session.add(Course(name="C", codename="PB071"))
    db.session.commit()


def create_roles(db):
    cpp = Course.query.filter_by(codename="PB161").first()
    db.session.add(Role(name='student', course=cpp))
    db.session.add(Role(name='teacher', course=cpp))
    db.session.commit()


def create_projects(db):
    db.session.add(Project(course=Course.query.all()[0], name="p1", test_files_source="repo"))
    db.session.add(Project(course=Course.query.all()[1], name="p2", test_files_source="repo2"))
    db.session.commit()


def create_groups(db):
    db.session.add(Group(course=Course.query.filter_by(codename="PB161").first(), name="g1"))
    db.session.add(Group(course=Course.query.filter_by(codename="PB071").first(), name="g2"))
    db.session.commit()


def create_submissions(db):
    xfoo = User.query.filter_by(username="xfoo").first()
    xbar = User.query.filter_by(username="xbar").first()
    sub = Submission(user=xfoo, project=Project.query.filter_by(name="p1").first(), parameters="")
    sub.state = SubmissionState.IN_PROGRESS
    db.session.add(sub)
    db.session.add(Submission(user=xbar, project=Project.query.filter_by(name="p1").first(), parameters=""))
    db.session.commit()


def create_reviews(db):
    user1 = User.query.filter_by(username="xfoo").first()
    user2 = User.query.filter_by(username="xbar").first()
    review = Review(user1.submissions[0])
    ri1 = ReviewItem(user=user1, review=review, file="src/main.c", line=1, content="good stuff")
    ri2 = ReviewItem(user=user2, review=review, file="src/main.c", line=5, content="something")

    db.session.add(review)
    db.session.commit()


def create_relationships(db):
    user1 = User.query.filter_by(username="xfoo").first()
    cpp = Course.query.filter_by(codename="PB161").first()
    user1.roles.append(Role.query.filter_by(course=cpp).filter_by(name="student").first())
    user1.groups.append(Group.query.filter_by(course=cpp).filter_by(name="g1").first())
    db.session.commit()

